VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsDictionary"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'Copyright (C) 2004 Kristian. S.Stangeland

'This program is free software; you can redistribute it and/or
'modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; either version 2
'of the License, or (at your option) any later version.

'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.

'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' The word-type
Private Type Word
    sWord As String
    sDescription As String
End Type

' The header of the dictionary
Private Type Dictionary
    sName As String
    sAuthor As String
    sPassword As String
    dCreated As Date
End Type

' Events
Event Closing()
Event Changed(WordAmoutChanged As Boolean)
Event Opened()
Event Renewed()

' Public variables (accesible from other modules)
Public FileName As String
Public DictionaryChanged As Boolean

' Private variables
Private pHeader As Dictionary
Private pWords() As Word
Private pWordCount As Long

' ### Properties for the internal data ###
Public Property Get WordName(ByVal Index As Long) As String

    If IsValid(Index) Then
        ' Return the word
        WordName = pWords(Index).sWord
    End If

End Property

Public Property Let WordName(ByVal Index As Long, ByVal vNewValue As String)

    If IsValid(Index) Then
        ' Change the word to the passed string
        pWords(Index).sDescription = vNewValue
        
        ' The dictionary is changed
        ObjectChanged
    End If

End Property

Public Property Get WordDescription(ByVal Index As Long) As String

    If IsValid(Index) Then
        ' Return the description
        WordDescription = pWords(Index).sDescription
    End If

End Property

Public Property Let WordDescription(ByVal Index As Long, ByVal vNewValue As String)

    If IsValid(Index) Then
        ' Change the description to the passed string
        pWords(Index).sDescription = vNewValue
        
        ' The dictionary is changed
        ObjectChanged
    End If

End Property

Public Property Get WordCount() As Long

    WordCount = pWordCount

End Property

' ### Properties for the dictionary-header ###
Public Property Get DictionaryPassword() As String

    DictionaryPassword = pHeader.sPassword

End Property

Public Property Let DictionaryPassword(ByVal vNewValue As String)

    pHeader.sPassword = vNewValue
    
    ' The dictionary is changed
    ObjectChanged

End Property

Public Property Get DictionaryAuthor() As String

    DictionaryAuthor = pHeader.sAuthor

End Property

Public Property Let DictionaryAuthor(ByVal vNewValue As String)

    pHeader.sAuthor = vNewValue

    ' The dictionary is changed
    ObjectChanged

End Property

Public Property Get DictionaryName() As String

    DictionaryName = pHeader.sName

End Property

Public Property Let DictionaryName(ByVal vNewValue As String)

    pHeader.sName = vNewValue

    ' The dictionary is changed
    ObjectChanged

End Property

Public Property Get DictionaryCreated() As String

    DictionaryCreated = pHeader.dCreated

End Property

Public Property Let DictionaryCreated(ByVal vNewValue As String)

    pHeader.dCreated = vNewValue

    ' The dictionary is changed
    ObjectChanged

End Property

Public Function OpenDictionary(sFile As String) As Long

    On Error GoTo Error
    Dim Free As Long

    ' We first need to check whether the file exist
    If Dir(sFile) = "" Then
        ' If it dosen't exist, then we cannot open the dictionary
        OpenDictionary = -1
        Exit Function
    End If
    
    ' By using a handle, the application can access several files at once
    Free = FreeFile
    
    ' Then begin the process of reading the file
    Open sFile For Binary As #Free
        
        Get #Free, , pHeader
        Get #Free, , pWordCount
        
        ' Reallocate the array to the right size
        ReDim pWords(pWordCount)
        
        ' Read the entire array
        Get #Free, , pWords
        
    Close #Free
    
    ' We have successfully loaded the dictionary
    FileName = sFile
    
    ' Nothing is changed now, of course
    DictionaryChanged = False
    
    ' Inform about this
    RaiseEvent Opened
    
    Exit Function
    ' In case of an error
Error:

    ' We didn't manage to open
    OpenDictionary = -1
    
End Function

Public Function SaveDictionary(ByVal sFile As String) As Long

    On Error GoTo Error
    Dim Free As Long

    ' We first need to check whether the file exist
    If Dir(sFile) <> "" Then
        ' If it dose exist, then we must delete it
        Kill sFile
    End If
    
    ' By using a handle, the application can access several files at once
    Free = FreeFile
    
    ' Then begin the process of writing the dictionary to the file
    Open sFile For Binary As #Free

        Put #Free, , pHeader
        Put #Free, , pWordCount
        Put #Free, , pWords

    Close #Free

    ' Nothing is changed now
    DictionaryChanged = False

    Exit Function
    ' In case of an error
Error:

    ' We didn't manage to save
    SaveDictionary = -1

End Function

Public Sub CloseDictionary()

    ' Inform about the closing
    RaiseEvent Closing

    Erase pWords
    
    ' Clear the header
    With pHeader
        .dCreated = 0
        .sAuthor = ""
        .sName = ""
        .sPassword = ""
    End With
    
    ' We've cleared the array
    pWordCount = -1
    
    ' No file is open
    FileName = ""
    
    ' No changes has been made by now
    DictionaryChanged = False

End Sub

Public Sub AddWord(sWord As String, sDescription As String)

    ' Increse the count
    pWordCount = pWordCount + 1

    ' Reallocate the array
    ReDim Preserve pWords(pWordCount)
    
    ' Write the data
    pWords(pWordCount).sWord = sWord
    pWords(pWordCount).sDescription = sDescription
    
    ' The dictionary is changed
    ObjectChanged True
 
End Sub

Public Sub DeleteWord(ByVal Index As Long)

    Dim Tell As Long
    
    ' Move all elements one step down to the index
    For Tell = Index To pWordCount - 1
        LSet pWords(Tell) = pWords(Tell + 1)
    Next

    ' Decrese the count
    pWordCount = pWordCount - 1
    
    ' Resize the array
    If pWordCount < 0 Then
        ' If there's no elements left
        Erase pWords
    Else
        ReDim Preserve pWords(pWordCount)
    End If
    
    ' The dictionary is changed
    ObjectChanged True

End Sub

Public Sub FillComboBox(ComboBox As ComboBox)

    Dim Tell As Long
    
    For Tell = 0 To pWordCount
        ComboBox.AddItem pWords(Tell).sWord
    Next

End Sub

Public Sub CreateDictionary(Optional sPassword As String, Optional sAuthor As String, Optional sName As String)

    ' Close the current dictionary
    CloseDictionary
    
    ' Set the header
    With pHeader
        .dCreated = Now
        .sAuthor = sAuthor
        .sName = sName
        .sPassword = sPassword
    End With
    
    ' No words has been added yet
    pWordCount = -1

    ' Raise the expected event
    RaiseEvent Renewed

End Sub

Private Function IsValid(ByVal Index As Long) As Boolean

    IsValid = CBool(Index >= 0 And Index <= pWordCount)

End Function

Private Sub ObjectChanged(Optional AmoutChanged As Boolean)
    
    RaiseEvent Changed(AmoutChanged)
    DictionaryChanged = True

End Sub

Private Sub Class_Initialize()

    ' We always start with a fresh dictionary
    CreateDictionary
    
End Sub

Private Sub Class_Terminate()

    ' Close this dictionary
    CloseDictionary

End Sub
